@@ -17,8 +17,8 @@ no warnings "experimental::lexical_subs";
 
 package  Data::Dumper::Interp;
 { no strict 'refs'; ${__PACKAGE__."::VER"."SION"} = 997.999; }
-our $VERSION = '6.006'; # VERSION from Dist::Zilla::Plugin::OurPkgVersion
-our $DATE = '2023-09-02'; # DATE from Dist::Zilla::Plugin::OurDate
+our $VERSION = '6.007'; # VERSION from Dist::Zilla::Plugin::OurPkgVersion
+our $DATE = '2023-09-18'; # DATE from Dist::Zilla::Plugin::OurDate
 
 package
   # newline so Dist::Zilla::Plugin::PkgVersion won't add $VERSION
@@ -179,10 +179,18 @@ sub oops(@) { @_=("\n".(caller)." oops:\n",@_,"\n"); goto &Carp::confess }
 sub btwN($@) { my $N=shift; local $_=join("",@_); s/\n\z//s; printf "%4d: %s\n",(caller($N))[2],$_; }
 sub btw(@) { unshift @_,0; goto &btwN }
 
-sub __chop_loc($) {  # remove "at ..." from an exception message
+sub _chop_ateval($) {  # remove "at (eval N) line..." from an exception message
   (local $_ = shift) =~ s/ at \(eval[^\)]*\) line \d+[^\n]*\n?\z//s;
   $_
 }
+sub _croak_or_confess(@) {
+  # Chain to croak, or to confess if there is an eval in the call stack
+  if (Carp::longmess("") =~ /\beval\b/) {
+    goto &Carp::confess;
+  }
+  goto &Carp::croak;
+}
+
 sub _tf($) { $_[0] ? "T" : "F" }
 sub _showfalse(_) { $_[0] ? $_[0] : 0 }
 sub _dbvisnew($) {
@@ -427,7 +435,7 @@ sub qshlist(@) { join " ", map{qsh} @_ }
 sub __getself { # Return $self if passed or else create a new object
   local $@;
   my $blessed = eval{ blessed($_[0]) }; # In case a tie handler throws
-  croak __chop_loc($@) if $@;
+  croak _chop_ateval($@) if $@;
   $blessed && $_[0]->isa(__PACKAGE__) ? shift : __PACKAGE__->new()
 }
 sub __getself_s { &__getself->Values([$_[0]]) }
@@ -451,7 +459,7 @@ sub _generate_sub($;$) {
   my ($arg, $proto_only) = @_;
   (my $methname = $arg) =~ s/.*:://;
   my sub error($) {
-    confess "Invalid sub/method name '$methname' (@_)\n"
+    _croak_or_confess "Invalid sub/method name '$methname' (@_)\n"
   }
 
   # Method names are ivis, dvis, vis, avis, or hvis with prepended
@@ -476,7 +484,7 @@ sub _generate_sub($;$) {
   if ($basename =~ /^[id]/) {
     error "'$1' is inapplicable to $basename" if /([ahl])/;
   }
-  error "'$1' mis-placed: Only allowed as '${1}vis'" if /([ahid])/;
+  error "'$1' mis-placed: Only allowed as '${1}vis'" if /([ahi])/;
 
 
   # All these subs can be called as either or methods or functions.
@@ -519,6 +527,7 @@ sub _generate_sub($;$) {
     $code .= '->Useqq(0)'     if delete $mod{q};
     $code .= '->Useqq("unicode:controlpics")' if delete $mod{c};
     $code .= '->Refaddr(1)'   if delete $mod{r};
+    $code .= '->Debug(2)'     if delete $mod{d};
 
     if ($basename =~ /^([id])vis/) {
       $code .= ", shift, '$1' ); goto &_Interpolate }";
@@ -621,6 +630,9 @@ sub _Do {
   my $original = $orig_values[0];
   btw "##ORIGINAL=",u($original),"=",_dbvis($original) if $debug;
 
+  _croak_or_confess "*vis($original) called in void context.\nDid you forget to 'say ...'?"
+    if ! defined wantarray;
+
   # Allow one extra level if we wrapped the user's args in __getself_[ah]
   $my_maxdepth = $self->Maxdepth || INT_MAX;
   ++$my_maxdepth if $listform && $my_maxdepth < INT_MAX;
@@ -635,7 +647,7 @@ sub _Do {
   # maximally-compact string, and then manually fold the result to Foldwidth,
   # inserting the user's Pad before each line *except* the first.
   #
-  # Also disable Maxdepth because we did it ourself (see visit_ref).
+  # Also disable Maxdepth because we handle that ourself (see visit_ref).
   my $users_Maxdepth = $self->Maxdepth; # implemented by D::D
   $self->Maxdepth(0);
   my $users_pad = $self->Pad();
@@ -1640,6 +1652,9 @@ sub _postprocess_DD_result {
 
 sub _Interpolate {
   my ($self, $input, $i_or_d) = @_;
+  _croak_or_confess $i_or_d."vis('$input') called in void context.\nDid you forget to 'say ...'?"
+    unless defined wantarray;
+
   return "<undef arg>" if ! defined $input;
 
   &_SaveAndResetPunct;
@@ -1652,7 +1667,7 @@ sub _Interpolate {
 
   my @pieces;  # list of [visfuncname or 'p' or 'e', inputstring]
   { local $_ = $input;
-    if (/\b((?:ARRAY|HASH)\(0x[a-fA-F0-9]+\))/) {
+    if (/\b((?:ARRAY|HASH|SCALAR)\(0x[a-fA-F0-9]+\))/) {
       state $warned=0;
       carp("Warning: String passed to $funcname may have been interpolated by Perl\n(use 'single quotes' to avoid this)\n") unless $warned++;
     }
@@ -1704,7 +1719,7 @@ sub _Interpolate {
         if ($i_or_d eq 'd') {
           # Inject a "plain text" fragment containing the "expr=" prefix,
           # omitting the '$' sigl if the expr is a plain '$name'.
-          push @pieces, ['p', (/^\$(?!_)(${userident_re})\z/ ? $1 : $_)."="];
+          push @pieces, ['P', (/^\$(?!_)(${userident_re})\z/ ? $1 : $_)."="];
         }
         if ($sigl eq '$') {
           push @pieces, ["vis", $_];
@@ -1715,13 +1730,16 @@ sub _Interpolate {
         elsif ($sigl eq '%') {
           push @pieces, ["hvis", $_];
         }
-        else { confess "BUG:sigl='$sigl'"; }
-      } else {
-        if (/^.+?(?<!\\)([\$\@\%])/) { confess __PACKAGE__." bug: Missed '$1' in «$_»" }
+        else { oops }
+      }
+      else {
+        if (/^.+?(?<!\\)([\$\@\%])/) {
+          confess __PACKAGE__." bug: Missed '$1' in «$_»"
+        }
         # Due to the need to simplify the big regexp above, \x{abcd} is now
         # split into "\x" and "{abcd}".  Combine consecutive pass-thrus
-        # into a single passthru ('p') and convert later to 'e' if
-        # an eval if needed.
+        # into a single passthru ('p'), converted later to 'e' if an eval
+        # is needed.
         if (@pieces && $pieces[-1]->[0] eq 'p') {
           $pieces[-1]->[1] .= $_;
         } else {
@@ -1744,11 +1762,38 @@ sub _Interpolate {
     }
     foreach (@pieces) {
       my ($meth, $str) = @$_;
-      next unless $meth eq 'p' && $str =~ /\\[abtnfrexXN0-7]/;
-      $str =~ s/([()\$\@\%])/\\$1/g;  # don't hide \-escapes to be interpolated!
-      $str =~ s/\$\\/\$\\\\/g;
-      $_->[1] = "qq(" . $str . ")";
-      $_->[0] = 'e';
+      # If the user uses 'single quoted' strings then backslash escapes
+      # can not be emulated exactly as they would work in double-quoted strings
+      # because \ is inconsistently passed through, namely only when not
+      # followed by another backslash (or a quote character).
+      #   say ivis '\015';   # octal escape for CR intended?
+      #   say ivis '\\015';  # four literal characters \015 intended?
+      # We can not tell the difference because we get \015 in both cases.
+      #
+      # Currently we interpolate all \-escapes we see, so to get a literal
+      # backslash users must double them, e.g.
+      #   say ivis 'The four char escape sequence \\\\015 produces \015';
+      # Here-docs do not treat \ specially and so avoid this problem:
+      #   say ivis <<\END;
+      #   The four char escape sequence \\015 produces \015
+      #   END
+      #
+      # 0/18/23: Now really *all* \-escapes are interpolated, so this works:
+      #   say ivis '\$foo = $foo'   # $foo = <value>
+
+      #next unless $meth eq 'p' && $str =~ /\\[abtnfrexXN0-7]/;
+      #$str =~ s/([()\$\@\%])/\\$1/g;  # dont hide \-escapes to be interpolated!
+
+      if ($meth eq 'p') {
+        if ($str =~ /\\./) {
+          $str =~ s/\$\\/\$\\\\/g;   # Assume the punct var $\ is not intended
+          $_->[1] = "qq(" . $str . ")";
+          $_->[0] = 'e';
+        }
+      }
+      elsif ($meth eq 'P') {
+          $_->[0] = 'p';
+      }
     }
   } #local $_
 
@@ -1771,6 +1816,7 @@ sub DB_Vis_Interpolate {
   my $result = "";
   foreach my $p (@$pieces) {
     my ($methname, $arg) = @$p;
+#say "III methname=$methname arg='$arg'";
     if ($methname eq 'p') {
       $result .= $arg;
     }
@@ -1838,14 +1884,15 @@ sub DB_Vis_Eval($$) {
      .' @Data::Dumper::Interp::result = '.$evalarg.';'
      .' $Data::Dumper::Interp::save_stack[-1]->[0] = $@;' # possibly changed by a tie handler
      ;
+     ###??? FIXME why is DB_Vis_Evalwrapper needed?  Lexical scope?
      &DB_Vis_Evalwrapper;
      @Data::Dumper::Interp::result
   };
   my $errmsg = $@;
 
   if ($errmsg) {
-    $errmsg = Data::Dumper::Interp::__chop_loc($errmsg);
-    Carp::carp("${label_for_errmsg}: Error interpolating '$evalarg':\n$errmsg\n");
+    $errmsg = Data::Dumper::Interp::_chop_ateval($errmsg);
+    Carp::carp("${label_for_errmsg} interpolation error: $errmsg\n");
     @result = ( (defined($result[0]) ? $result[0] : "")."<invalid/error>" );
   }
 
